# pylint: disable=C0114,C0116

import os
from pathlib import Path
from shutil import copyfile

import pytest
from scripttest import TestFileEnvironment

FIXTURE_PATH = Path(os.getcwd(), "tests", "fixtures")


# See https://stackoverflow.com/a/62055409
@pytest.fixture(autouse=True)
def _change_test_dir(monkeypatch, tmp_path):
    monkeypatch.chdir(tmp_path)


@pytest.fixture(autouse=True, name="csv_file")
def fixture_csv_file(tmpdir):
    def csv_file(filename):
        src = Path(FIXTURE_PATH, filename)
        des = Path(tmpdir, src.name)
        copyfile(src, des)
        return des

    return csv_file


@pytest.fixture(autouse=True, name="cli_runner")
def fixture_cli_runner(tmpdir):
    def cli_runner(*args, **kwargs):
        cwd = tmpdir / "scripttest"
        env = TestFileEnvironment(cwd)
        kwargs.update({"cwd": cwd, "expect_error": True})
        return env.run("fotolab", *args, **kwargs)

    return cli_runner



# Copyright (C) 2024 Kian-Meng Ang
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

"""Auto subcommand."""

import argparse
import logging

import fotolab.contrast
import fotolab.resize
import fotolab.sharpen
import fotolab.watermark

log = logging.getLogger(__name__)


def build_subparser(subparsers) -> None:
    """Build the subparser."""
    auto_parser = subparsers.add_parser(
        "auto", help="auto adjust (resize, contrast, and watermark) a photo"
    )

    auto_parser.set_defaults(func=run)

    auto_parser.add_argument(
        dest="image_filename",
        help="set the image filename",
        type=str,
        default=None,
        metavar="IMAGE_FILENAME",
    )


def run(args: argparse.Namespace) -> None:
    """Run auto subcommand.

    Args:
        config (argparse.Namespace): Config from command line arguments

    Returns:
        None
    """
    extra_args = {
        "width": 600,
        "height": 277,
        "cutoff": 1,
        "radius": 1,
        "percent": 100,
        "threshold": 2,
        "text": "kianmeng.org",
        "position": "bottom-left",
        "font_size": 12,
        "font_color": "white",
        "outline_width": 2,
        "outline_color": "black",
    }
    combined_args = argparse.Namespace(**vars(args), **extra_args)
    combined_args.overwrite = True
    combined_args.open = False
    log.debug(args)
    log.debug(combined_args)

    fotolab.resize.run(combined_args)
    fotolab.contrast.run(combined_args)
    fotolab.sharpen.run(combined_args)
    fotolab.watermark.run(combined_args)



# Copyright (C) 2024 Kian-Meng Ang
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

"""Resize subcommand."""

import argparse
import logging

from PIL import Image

from fotolab import save_image

log = logging.getLogger(__name__)


def build_subparser(subparsers) -> None:
    """Build the subparser."""
    resize_parser = subparsers.add_parser("resize", help="resize an image")

    resize_parser.set_defaults(func=run)

    resize_parser.add_argument(
        dest="image_filename",
        help="set the image filename",
        type=str,
        default=None,
        metavar="IMAGE_FILENAME",
    )

    resize_parser.add_argument(
        "-wh",
        "--width",
        dest="width",
        help="set the width of the image (default: '%(default)s')",
        type=int,
        default="600",
        metavar="WIDTH",
    )

    resize_parser.add_argument(
        "-ht",
        "--height",
        dest="height",
        help="set the height of the image (default: '%(default)s')",
        type=int,
        default="277",
        metavar="HEIGHT",
    )


def run(args: argparse.Namespace) -> None:
    """Run resize subcommand.

    Args:
        args (argparse.Namespace): Config from command line arguments

    Returns:
        None
    """
    log.debug(args)

    original_image = Image.open(args.image_filename)
    resized_image = original_image.copy()
    resized_image = resized_image.resize(
        (args.width, args.height), Image.Resampling.LANCZOS
    )

    save_image(args, resized_image)



# Copyright (C) 2024 Kian-Meng Ang
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

"""Watermark subcommand."""

import argparse
import logging

from PIL import Image, ImageColor, ImageDraw, ImageFont

from fotolab import save_image

log = logging.getLogger(__name__)

POSITIONS = ["top-left", "top-right", "bottom-left", "bottom-right"]


def build_subparser(subparsers) -> None:
    """Build the subparser."""
    watermark_parser = subparsers.add_parser(
        "watermark", help="watermark an image"
    )

    watermark_parser.set_defaults(func=run)

    watermark_parser.add_argument(
        dest="image_filename",
        help="set the image filename",
        type=str,
        default=None,
        metavar="IMAGE_FILENAME",
    )

    watermark_parser.add_argument(
        "-t",
        "--text",
        dest="text",
        help="set the watermark text (default: '%(default)s')",
        type=str,
        default="kianmeng.org",
        metavar="WATERMARK_TEXT",
    )

    watermark_parser.add_argument(
        "-p",
        "--position",
        dest="position",
        choices=POSITIONS,
        help="set position of the watermark text (default: '%(default)s')",
        default="bottom-left",
    )

    watermark_parser.add_argument(
        "-fs",
        "--font-size",
        dest="font_size",
        type=int,
        default=12,
        help="set the font size of watermark text (default: '%(default)s')",
        metavar="FONT_SIZE",
    )

    watermark_parser.add_argument(
        "-fc",
        "--font-color",
        dest="font_color",
        type=str,
        default="white",
        help="set the font color of watermark text (default: '%(default)s')",
        metavar="FONT_COLOR",
    )

    watermark_parser.add_argument(
        "-ow",
        "--outline-width",
        dest="outline_width",
        type=int,
        default=2,
        help=(
            "set the outline width of the watermark text "
            "(default: '%(default)s')"
        ),
        metavar="OUTLINE_WIDTH",
    )

    watermark_parser.add_argument(
        "-oc",
        "--outline-color",
        dest="outline_color",
        type=str,
        default="black",
        help=(
            "set the outline color of the watermark text "
            "(default: '%(default)s')"
        ),
        metavar="OUTLINE_COLOR",
    )


def run(args: argparse.Namespace) -> None:
    """Run watermark subcommand.

    Args:
        config (argparse.Namespace): Config from command line arguments

    Returns:
        None
    """
    log.debug(args)

    original_image = Image.open(args.image_filename)
    watermarked_image = original_image.copy()

    draw = ImageDraw.Draw(watermarked_image)

    font = ImageFont.truetype("arial.ttf", args.font_size)

    (left, top, right, bottom) = draw.textbbox(
        xy=(0, 0), text=args.text, font=font
    )
    text_width = right - left
    text_height = bottom - top
    (position_x, position_y) = calculate_position(
        watermarked_image, text_width, text_height, args.position
    )

    draw.text(
        (position_x, position_y),
        args.text,
        font=font,
        fill=ImageColor.getrgb(args.font_color),
        stroke_width=args.outline_width,
        stroke_fill=ImageColor.getrgb(args.outline_color),
    )

    save_image(args, watermarked_image)


def calculate_position(image, text_width, text_height, position) -> tuple:
    """Calculate the boundary coordinates of the watermark text."""
    padding = 10
    if position == "top-left":
        position_x = 0 + padding
        position_y = 0 + padding
    elif position == "top-right":
        position_x = image.width - text_width - padding
        position_y = 0 + padding
    elif position == "bottom-left":
        position_x = 0 + padding
        position_y = image.height - text_height - padding
    elif position == "bottom-right":
        position_x = image.width - text_width - padding
        position_y = image.height - text_height - padding

    return (position_x, position_y)



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

from datetime import datetime

import fotolab

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

years = ", ".join([str(y) for y in range(2024, datetime.now().year + 1)])
project = "fotolab"
copyright = f"{years} Kian-Meng Ang"
author = "Kian-Meng Ang"

# The full version, including alpha/beta/rc tags
release = fotolab.__version__

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    "myst_parser",
    "sphinx_copybutton",
    "sphinx.ext.autodoc",
    "sphinx_autodoc_typehints",
    "sphinx.ext.napoleon",
]

templates_path = ["_templates"]
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = "alabaster"
html_static_path = ["_static"]
html_theme_options = {
    "logo": "logo.jpg",
    "description": "A console program that manipulates photos.",
    "github_user": "kianmeng",
    "github_repo": "fotolab",
    "github_banner": True,
    "github_button": True,
    "pre_bg": "#eee",
    "page_width": "1080px",
}



# Copyright (C) 2023 Kian-Meng Ang
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

"""Main entry point when running as module."""

from fotolab.cli import main

if __name__ == "__main__":
    main()
    raise SystemExit()



# Copyright (C) 2024 Kian-Meng Ang
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

"""Env subcommand."""

import argparse
import logging
import platform
import sys

from fotolab import __version__

log = logging.getLogger(__name__)


def build_subparser(subparsers) -> None:
    """Build the subparser."""
    env_parser = subparsers.add_parser(
        "env", help="print environment information for bug reporting"
    )

    env_parser.set_defaults(func=run)


def run(args: argparse.Namespace) -> None:
    """Run env subcommand.

    Args:
        config (argparse.Namespace): Config from command line arguments

    Returns:
        None
    """
    log.debug(args)

    sys_version = sys.version.replace("\n", "")

    print(f"fotolab: {__version__}")
    print(f"python: {sys_version}")
    print(f"platform: {platform.platform()}")



# Copyright (c) 2024 Kian-Meng Ang

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Generals Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

"""A console program to manipulate photos.

  website: https://github.com/kianmeng/fotolab
  changelog: https://github.com/kianmeng/fotolab/blob/master/CHANGELOG.md
  issues: https://github.com/kianmeng/fotolab/issues
"""

import argparse
import logging
import sys
from typing import Dict, Optional, Sequence

import fotolab.auto
import fotolab.border
import fotolab.contrast
import fotolab.env
import fotolab.resize
import fotolab.sharpen
import fotolab.watermark
from fotolab import __version__

log = logging.getLogger(__name__)


def setup_logging(args: argparse.Namespace) -> None:
    """Set up logging by level."""
    if args.quiet:
        logging.disable(logging.NOTSET)
    else:
        conf: Dict = {
            True: {
                "level": logging.DEBUG,
                "msg": "[%(asctime)s] %(levelname)s: %(name)s: %(message)s",
            },
            False: {"level": logging.INFO, "msg": "%(message)s"},
        }

        logging.basicConfig(
            level=conf[args.debug]["level"],
            stream=sys.stdout,
            format=conf[args.debug]["msg"],
            datefmt="%Y-%m-%d %H:%M:%S",
        )


def build_parser() -> argparse.ArgumentParser:
    """Build the CLI parser."""
    parser = argparse.ArgumentParser(
        description=__doc__,
        formatter_class=lambda prog: argparse.RawTextHelpFormatter(
            prog, max_help_position=20
        ),
    )

    parser.add_argument(
        "-o",
        "--overwrite",
        default=False,
        action="store_true",
        dest="overwrite",
        help="overwrite existing image",
    )

    parser.add_argument(
        "-op",
        "--open",
        default=False,
        action="store_true",
        dest="open",
        help="open the image using default program (default: '%(default)s'",
    )

    parser.add_argument(
        "-od",
        "--output-dir",
        dest="output_dir",
        default="output",
        help="set default output folder (default: '%(default)s')",
    )

    parser.add_argument(
        "-q",
        "--quiet",
        default=False,
        action="store_true",
        dest="quiet",
        help="suppress all logging",
    )

    parser.add_argument(
        "-d",
        "--debug",
        default=False,
        action="store_true",
        dest="debug",
        help="show debugging log and stacktrace",
    )

    parser.add_argument(
        "-V",
        "--version",
        action="version",
        version=f"fotolab {__version__}",
    )

    subparsers = parser.add_subparsers(help="sub-command help")
    fotolab.auto.build_subparser(subparsers)
    fotolab.border.build_subparser(subparsers)
    fotolab.contrast.build_subparser(subparsers)
    fotolab.resize.build_subparser(subparsers)
    fotolab.sharpen.build_subparser(subparsers)
    fotolab.watermark.build_subparser(subparsers)
    fotolab.env.build_subparser(subparsers)

    return parser


def main(args: Optional[Sequence[str]] = None) -> None:
    """Run the main program flow."""
    args = args or sys.argv[1:]
    parsed_args = argparse.Namespace(debug=True)
    log.debug(args)

    try:
        parser = build_parser()
        if len(args) == 0:
            parser.print_help(sys.stderr)
        else:
            parsed_args = parser.parse_args(args)
            setup_logging(parsed_args)

            if hasattr(parsed_args, "func"):
                log.debug(parsed_args)
                parsed_args.func(parsed_args)
            else:
                parser.print_help(sys.stderr)

    except Exception as error:
        log.error(
            "error: %s",
            getattr(error, "message", str(error)),
            exc_info=getattr(parsed_args, "debug", True),
        )

        raise SystemExit(1) from None
